
#include "stdafx.h"
#include "serial.h"
#include "lcd.h"

unsigned char lcdbuffer[480][640];

const signed char SIN[64]=
{0, 12, 24, 37, 48, 60, 71, 81, 90, 98, 106, 112, 118, 122, 125, 127,
 127, 127, 125, 122, 118, 112, 106, 98, 90, 81, 71, 60, 48, 37, 24, 12,
 0,-12,-24,-37,-48,-60,-71,-81,-90,-98,-106,-112,-118,-122,-125,-127,
-127,-127,-125,-122,-118,-112,-106,-98,-90,-81,-71,-60,-48,-37,-24,-12};

const signed char COS[64]=
{127, 127, 125, 122, 118, 112, 106, 98, 90, 81, 71, 60, 48, 37, 24, 12,
 0,-12,-24,-37,-48,-60,-71,-81,-90,-98,-106,-112,-118,-122,-125,-127,
-127,-127,-125,-122,-118,-112,-106,-98,-90,-81,-71,-60,-48,-37,-24,-12,
 0, 12, 24, 37, 48, 60, 71, 81, 90, 98, 106, 112, 118, 122, 125, 127};



void writebyte (unsigned char wert)
{
	if (wert==Escape)
		SendByte(Escape);
	SendByte(wert);
}

void writecom (unsigned char wert)
{
	SendByte(Escape);
	if (wert==Escape)
		SendByte(Escape);
	SendByte(wert);
}

void setpwm (unsigned char wert)
{
	unsigned char temp[5],i=0;
	temp[i++]=Escape;
	temp[i++]=2;
	if (wert==Escape)
		temp[i++]=Escape;
	temp[i++]=wert;
	SendString(temp, i);
}

void setscroll (unsigned char wert)
{
	unsigned char temp[5],i=0;
	temp[i++]=Escape;
	temp[i++]=3;
	if (wert==Escape)
		temp[i++]=Escape;
	temp[i++]=wert;
	SendString(temp, i);
}

void setcursor (unsigned int x, unsigned int y)
{
	unsigned char temp[10],i=0;
	temp[i++]=Escape;
	temp[i++]=1;
	if (((char)(y%240))==Escape)
		temp[i++]=Escape;
	temp[i++]=((char)(y%240));
	if (((char)((y/240)*128+(x/8)))==Escape)
		temp[i++]=Escape;
	temp[i++]=((char)((y/240)*128+(x/8)));
	SendString(temp, i);
}

void writetext (char *text)
{
	while (*text)
	{	writecom(192);
		writebyte(*text++);
	}
}

void initlcd(void)
{
	Serial_Init("COM5", 1000000);		//COM5, 1MBaud

	setpwm(75);	//Kontrast einstellen

	setscroll(0);	

	clearbuffer();

	transfer(0,0,640,480);
}

#define color 1

void lcd_set_pixel (int x, int y)
{	if ((x<0)||(x>639))
		return;
	if ((y<0)||(y>479))
		return;
	lcdbuffer[y][x]=color;
}


void lcd_line (int x1, int y1, int x2, int y2)
{   int temp;
    if (y1==y2)
       {if (x1>x2)
            {temp=x1;
            x1=x2;
            x2=temp;}
        h_line (x1, x2, y1);}
    else if (x1==x2)
       {if (y1>y2)
            {temp=y1;
            y1=y2;
            y2=temp;}
        v_line (x1, y1, y2); }
    else
        bresenham_line (x1, y1, x2, y2);
}

void bresenham_line (int x1, int y1, int x2, int y2)
   {int dx, dy, x, y, temp;
    int IncrE, IncrNE, d;
      if (y1>y2)
       {temp=y1;
        y1=y2;
        y2=temp;
        temp=x1;
        x1=x2;
        x2=temp;}
    dy = y2-y1;
    if (x1>x2)
       {dx = x1-x2;
        if (dy>dx)
           {IncrE = 2*dx;
            IncrNE = 2*(dx-dy);
            d = IncrE -2*dy;
            x = x1;
            lcd_set_pixel (x,y1);
            for (y=y1+1; y<y2; y++)
               {if (d<=0)
                    d+=IncrE;
                else
                   {d+=IncrNE;
                    x--; }
                lcd_set_pixel (x,y); }}
        else
           {IncrE = 2*dy;
            IncrNE = 2*(dy-dx);
            d = IncrE -2*dx;
            y = y1;
            lcd_set_pixel (x1,y);
            for (x=x1-1; x>x2; x--)
               {if (d<=0)
                    d+=IncrE;
                else
                   {d+=IncrNE;
                    y++; }
                lcd_set_pixel (x,y); }}}
    else
       {dx = x2-x1;
        if (dy>dx)
           {IncrE = 2*dx;
            IncrNE = 2*(dx-dy);
            d = IncrE -2*dy;
            x = x1;
            lcd_set_pixel (x,y1);
            for (y=y1+1; y<y2; y++)
               {if (d<=0)
                    d+=IncrE;
                else
                   {d+=IncrNE;
                    x++; }
             lcd_set_pixel (x,y); }}
        else
           {IncrE = 2*dy;
            IncrNE = 2*(dy-dx);
            d = IncrE -2*dx;
            y = y1;
            lcd_set_pixel (x1,y);
            for (x=x1+1; x<x2; x++)
               {if (d<=0)
                    d+=IncrE;
                else
                   {d+=IncrNE;
                    y++; }
             lcd_set_pixel (x,y); }}}
}

void h_line (int x1, int x2, int y)
   {int x;
    for (x=x1; x<=x2; x++)
	lcd_set_pixel(x,y);
}

void v_line (int x, int y1, int y2)
   {int y;
    for (y=y1; y<=y2; y++)
	lcd_set_pixel(x,y);
}

void transfer (int xmin, int ymin, int xmax, int ymax)
{   int y,x,i,j=0;
	unsigned char c;
	unsigned char temp[250];

	for (y=ymin; y<ymax; y++)
	{	setcursor (xmin, y);
		j=0;
		for (x=xmin; x<xmax; x+=8)
		{c=0;
			for (i=0; i<8;i++)
				c+=lcdbuffer[y][x+i]*(1<<(7-i));
			if (c==Escape)
				temp[j++]=Escape;
			temp[j++]=c;
		}
		SendString(temp, j);
	}
}


void clearbuffer (void)
{   unsigned int y,x;

	for (y=0; y<480; y++)
		for (x=0; x<640; x++)
		lcdbuffer[y][x]=1-color;
}

int M[3][3];

void Make_Matrix(unsigned char W1,unsigned char W2,unsigned char W3) 
{signed char cosW1, cosW2, cosW3, sinW1, sinW2, sinW3;

    sinW1=SIN[W1];
    sinW2=SIN[W2];
    sinW3=SIN[W3];
    cosW1=COS[W1];
    cosW2=COS[W2];
    cosW3=COS[W3];

  // Rotation-Matrix 
  // calculates the new position of the single vectors
  M[0][0] = ( cosW2*cosW3)/64;
  M[1][0] = (-cosW2*sinW3)/64;
  M[2][0] =   sinW2*2;

  M[0][1] = (cosW1*sinW3)/64  + (((sinW1*sinW2)/256)*cosW3)/32;
  M[1][1] = (cosW1*cosW3)/64  - (((sinW1*sinW2)/256)*sinW3)/32;
  M[2][1] =(-sinW1*cosW2)/64;

  M[0][2] = (sinW1*sinW3)/64  - (((cosW1*sinW2)/256)*cosW3)/32;
  M[1][2] = (sinW1*cosW3)/64  + (((cosW1*sinW2)/256)*sinW3)/32;
  M[2][2] = (cosW1*cosW2)/64;
}

#define MUL 256
#define x_Pos 320
#define y_Pos 240

void lcd_line3d (int x1,int y1,int z1, int x2,int y2,int z2, int W1, int W2, int W3)
{int xr1,xr2 ;
 int yr1,yr2 ;

  Make_Matrix ((char)W1,(char)W2,(char)W3);
 
    // Convert the vector through the Rotation-Matrix
    xr1 = (M[0][0] * x1 + M[1][0]*y1 + M[2][0]*z1)/MUL;
    yr1 = (M[0][1] * x1 + M[1][1]*y1 + M[2][1]*z1)/MUL;
    xr2 = (M[0][0] * x2 + M[1][0]*y2 + M[2][0]*z2)/MUL;
    yr2 = (M[0][1] * x2 + M[1][1]*y2 + M[2][1]*z2)/MUL;

    // Draws the line
    if (((xr1+x_Pos)>0)&&((xr2+x_Pos)>0)&&((yr1+y_Pos)>0)&&((yr1+y_Pos)>0))
        lcd_line( xr1 + x_Pos, yr1 + y_Pos, xr2 + x_Pos, yr2 + y_Pos);
}


